@@ -1,7 +1,7 @@
 #!/usr/bin/python
 #
 # Copyright 2005 Lars Wirzenius (liw@iki.fi)
-# Copyright 2009 Holger Levsen (holger@layer-acht.org)
+# Copyright 2009-2010 Holger Levsen (holger@layer-acht.org)
 # 
 # This program is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -31,8 +31,8 @@ import logging
 import ConfigParser
 import urllib
 import shutil
-import string
 import re
+import string
 
 # if python-rpy ain't installed, we don't draw fancy graphs
 try:
@@ -49,7 +49,9 @@ CONFIG_FILE = "/etc/piuparts/piuparts.conf"
 HTML_HEADER = """
 <!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
  <html><head><meta http-equiv="Content-Type" content="text/html; charset=utf-8">
-  <title>piuparts.debian.org / piuparts.cs.helsinki.fi</title>
+  <title>
+   $page_title
+  </title>
   <link type="text/css" rel="stylesheet" href="/style.css">
   <link rel="shortcut icon" href="/favicon.ico">
  </head>
@@ -78,21 +80,22 @@ HTML_HEADER = """
     </tr>
     <tr class="normalrow">
      <td class="contentcell">
-      <a href="/">Overview</a>
+      <a href="/">About + News</a>
      </td>
     </tr>
-      <td class="contentcell">
-      <a href="http://wiki.debian.org/piuparts" target="_blank">About</a>
+    <tr class="normalrow">
+     <td class="contentcell">
+      <a href="http://wiki.debian.org/piuparts" target="_blank">Overview</a>
      </td>
     </tr>
     <tr class="normalrow">
      <td class="contentcell">
       <a href="http://wiki.debian.org/piuparts/FAQ" target="_blank">FAQ</a> 
      </td>
-     </tr>     
+    </tr>     
     <tr class="normalrow">
      <td class="contentcell">
-      <a href="http://bugs.debian.org/piuparts" target="_blank">Bugs</a> / <a href="http://svn.debian.org/viewsvn/piuparts/trunk/TODO" target="_blank">ToDo</a>
+      <a href="http://bugs.debian.org/src:piuparts" target="_blank">Bugs</a> / <a href="http://anonscm.debian.org/gitweb/?p=piuparts/piuparts.git;a=blob;f=TODO" target="_blank">ToDo</a>
      </td>
     </tr>     
     <tr class="titlerow">
@@ -105,7 +108,6 @@ HTML_HEADER = """
       <a href="/doc/README.html" target="_blank">piuparts README</a>
      </td>
     </tr>
-    <tr class="titlerow">
     <tr class="normalrow">
      <td class="contentcell">
       <a href="/doc/piuparts.1.html" target="_blank">piuparts manpage</a>
@@ -113,7 +115,9 @@ HTML_HEADER = """
     </tr>
     <tr class="normalrow">
      <td class="contentcell">
-      <a href="http://svn.debian.org/viewsvn/piuparts/piatti/org/piuparts.debian.org/etc/piuparts.conf.piatti" target="_blank">piuparts.d.o configuration</a>
+      piuparts.d.o configuration:<br>
+      <a href="http://anonscm.debian.org/gitweb/?p=piuparts/piatti.git;a=blob;f=org/piuparts.debian.org/etc/piuparts.conf.piatti" target="_blank">piuparts.conf.piatti</a><br>
+      <a href="http://anonscm.debian.org/gitweb/?p=piuparts/piatti.git;a=tree;f=org/piuparts.debian.org/etc/scripts" target="_blank">scripts</a>
      </td>
     </tr>
     <tr class="normalrow">
@@ -128,7 +132,7 @@ HTML_HEADER = """
     </tr>
     <tr>
      <td class="contentcell">
-      <a href="http://bugs.debian.org/cgi-bin/pkgreport.cgi?tag=piuparts;users=debian-qa@lists.debian.org&archive=both" target="_blank">Bugs filed</a> 
+      <a href="http://bugs.debian.org/cgi-bin/pkgreport.cgi?tag=piuparts;users=debian-qa@lists.debian.org&amp;archive=both" target="_blank">Bugs filed</a> 
      </td>
     </tr>     
     $section_navigation
@@ -186,10 +190,10 @@ HTML_FOOTER = """
    GPL2 <a href="http://packages.debian.org/changelogs/pool/main/p/piuparts/current/copyright" target="_blank">licenced</a>.
    Weather icons are from the <a href="http://tango.freedesktop.org/Tango_Icon_Library" target="_blank">Tango Icon Library</a>.
    <a href="http://validator.w3.org/check?uri=referer">
-    <img border="0" src="/images/valid-html401.png" alt="Valid HTML 4.01!" height="15" width="80" valign="middle">
+    <img border="0" src="/images/valid-html401.png" alt="Valid HTML 4.01!" height="15" width="80" align="middle">
    </a>
    <a href="http://jigsaw.w3.org/css-validator/check/referer">
-    <img border="0" src="/images/w3c-valid-css.png" alt="Valid CSS!"  height="15" width="80" valign="middle">
+    <img border="0" src="/images/w3c-valid-css.png" alt="Valid CSS!"  height="15" width="80" align="middle">
    </a>
   </div>
  </div>
@@ -248,13 +252,13 @@ SECTION_INDEX_BODY_TEMPLATE = """
     </tr>
     <tr class="titlerow">
      <td class="alerttitlecell" colspan="3">
-      Packages per state
+      Binary packages per state
      </td>
     </tr>
     $tablerows
     <tr class="titlerow">
      <td class="titlecell" colspan="3">
-      URL to packages file(s)
+      URL to Packages file
      </td>
     </tr>
      <tr class="normalrow">
@@ -265,6 +269,19 @@ SECTION_INDEX_BODY_TEMPLATE = """
    </table>
 """
 
+MAINTAINER_BODY_TEMPLATE = """
+   <table class="righttable">
+    <tr class="titlerow">
+     <td class="titlecell" colspan="6">
+      $maintainer
+     </td>
+    </tr>
+    $distrolinks
+    $rows
+   </table>
+"""
+
+
 SOURCE_PACKAGE_BODY_TEMPLATE = """
    <table class="righttable">
     $rows
@@ -312,7 +329,6 @@ title_by_dir = {
     "pass": "PASSED piuparts logs",
     "fail": "Failed UNREPORTED piuparts logs",
     "bugged": "Failed REPORTED piuparts logs",
-    "fixed": "Failed and FIXED packages",
     "reserved": "RESERVED packages",
     "untestable": "UNTESTABLE packages",
 }
@@ -324,8 +340,6 @@ desc_by_dir = {
             "Bugs have not yet been reported.",
     "bugged": "Log files for packages that have FAILED testing. " +
               "Bugs have been reported, but not yet fixed.",
-    "fixed": "Log files for packages that have FAILED testing, but for " +
-             "which a fix has been made.",
     "reserved": "Packages that are RESERVED for testing on a node in a " +
                 "distributed piuparts network.",
     "untestable": "Log files for packages that have are UNTESTABLE with " +
@@ -336,11 +350,15 @@ state_by_dir = {
     "pass": "successfully-tested",
     "fail": "failed-testing",
     "bugged": "failed-testing",
-    "fixed": "fix-not-yet-tested",
     "reserved": "waiting-to-be-tested",
     "untestable": "dependency-cannot-be-tested",
 }
 
+# maybe better to use a list here to be able to use a defined order
+# see http://docs.python.org/library/bisect.html 2nd example
+# or use XX_name.tpl and get the linktarget from the template 
+# (its a substring of the <title> of the that template
+# the 2nd option should be preferend, maintaining this list is errorprone and tiresome
 linktarget_by_template = {
     "command_not_found_error.tpl": "due to a 'command not found' error",
     "files_in_usr_local_error.tpl": "due to files in /usr/local",
@@ -350,13 +368,24 @@ linktarget_by_template = {
     "processes_running_error.tpl": "due to leaving processes running behind",
     "unowned_files_after_purge_error.tpl": "due to unowned files after purge",
     "conffile_prompt_error.tpl": "due to prompting due to modified conffiles",
-    "unknown_purge_error.tpl": "dpkg --purge failed due to an unknown reason",
+    "unknown_purge_error.tpl": "due to purge failed due to an unknown reason",
     "insserv_error.tpl": "due to a problem with insserv",
-    "unknown_failures.tpl": "unclassified failures",
-    "initdscript_lsb_header_issue.tpl": "update-rc.d issues",
+    "unknown_failures.tpl": "due to unclassified failures",
+    "initdscript_lsb_header_issue.tpl": "but logfile contains update-rc.d issues",
     "command_not_found_issue.tpl": "but logfile contains 'command not found'",
     "broken_symlinks_issue.tpl": "but logfile contains 'broken symlinks'",
-    "broken_symlinks_error.tpl": "also logfile contains 'broken symlinks'",
+    "broken_symlinks_error.tpl": "...and logfile also contains 'broken symlinks'",
+    "pre_installation_script_error.tpl": "due to pre-installation maintainer script failed",
+    "post_installation_script_error.tpl": "due to post-installation maintainer script failed",
+    "pre_removal_script_error.tpl": "due to pre-removal maintainer script failed",
+    "post_removal_script_error.tpl": "due to post-removal maintainer script failed",
+    "cron_error_after_removal_error.tpl": "due to errors from cronjob after removal",
+    "logrotate_error_after_removal_error.tpl": "due to errors from logrotate after removal",
+    "problems_and_no_force_error.tpl": "due to not enough force being used",
+    "pre_depends_error.tpl": "due to a problem with pre-depends",
+    "modified_files_after_purge_error.tpl": "due to files having been modified after purge",
+    "disappeared_files_after_purge_error.tpl": "due to files having disappeared after purge",
+    "db_setup_error.tpl": "due to failing to setup a database",
 }
 
 
@@ -372,6 +401,7 @@ class Config(piupartslib.conf.Config):
                 "sources-url": None,
                 "master-directory": ".",
                 "description": "",
+                "known_circular_depends": "",
             }, "")
 
 
@@ -391,13 +421,13 @@ def setup_logging(log_level, log_file_name):
         logger.addHandler(handler)
 
 
-def html_protect(str):
-    str = "&amp;".join(str.split("&"))
-    str = "&lt;".join(str.split("<"))
-    str = "&gt;".join(str.split(">"))
-    str = "&#34;".join(str.split('"'))
-    str = "&#39;".join(str.split("'"))
-    return str
+def html_protect(vstr):
+    vstr = "&amp;".join(vstr.split("&"))
+    vstr = "&lt;".join(vstr.split("<"))
+    vstr = "&gt;".join(vstr.split(">"))
+    vstr = "&#34;".join(vstr.split('"'))
+    vstr = "&#39;".join(vstr.split("'"))
+    return vstr
 
 
 def emphasize_reason(reason):
@@ -427,8 +457,8 @@ def find_files_with_suffix(dir,suffix):
             files += [os.path.join(dir,subdir, name_in_subdir)]
     # sort by age
     content = {}
-    for file in files:
-      content[file] = os.path.getmtime(os.path.join(dir,file))
+    for vfile in files:
+      content[vfile] = os.path.getmtime(os.path.join(dir,vfile))
     # Sort keys, based on time stamps
     files = content.keys()
     files.sort(lambda x,y: cmp(content[x],content[y]))
@@ -444,21 +474,21 @@ def update_file(source, target):
 
 
 def copy_logs(logs_by_dir, output_dir):
-    for dir in logs_by_dir:
-        fulldir = os.path.join(output_dir, dir)
+    for vdir in logs_by_dir:
+        fulldir = os.path.join(output_dir, vdir)
         if not os.path.exists(fulldir):
             os.makedirs(fulldir)
-        for basename in logs_by_dir[dir]:
-            source = os.path.join(dir, basename)
+        for basename in logs_by_dir[vdir]:
+            source = os.path.join(vdir, basename)
             target = os.path.join(fulldir, basename)
             update_file(source, target)
 
 def remove_old_logs(logs_by_dir, output_dir):
-    for dir in logs_by_dir:
-        fulldir = os.path.join(output_dir, dir)
+    for vdir in logs_by_dir:
+        fulldir = os.path.join(output_dir, vdir)
         if os.path.exists(fulldir):
             for basename in os.listdir(fulldir):
-                if basename not in logs_by_dir[dir]:
+                if basename not in logs_by_dir[vdir]:
                     os.remove(os.path.join(fulldir, basename))
 
 
@@ -484,8 +514,8 @@ def create_section_navigation(section_names,current_section="sid"):
     for section in section_names:
         tablerows += ("<tr class=\"normalrow\"><td class=\"contentcell\"><a href='/%s'>%s</a></td></tr>\n") % \
                           (html_protect(section), html_protect(section))
-    tablerows += "<tr><td class=\"contentcell\"><a href=\"/%s/source/\">by source package</a></td></tr>" % current_section
     tablerows += "<tr><td class=\"contentcell\"><a href=\"/%s/maintainer/\">by maintainer / uploader</a></td></tr>" % current_section
+    tablerows += "<tr><td class=\"contentcell\"><a href=\"/%s/source/\">by source package</a></td></tr>" % current_section
     return tablerows;
 
 def get_email_address(maintainer):
@@ -505,12 +535,18 @@ class Section:
         self._config.read(CONFIG_FILE)
         logging.debug("-------------------------------------------")
         logging.debug("Running section " + self._config.section)
-        logging.debug("Loading and parsing Packages file")
 
+        known_circular_depends = []
+        for kcd in self._config["known_circular_depends"].split():
+          known_circular_depends.append(kcd)
+          #logging.debug("circular depends: " + kcd)
+
+        logging.debug("Loading and parsing Packages file")
         logging.info("Fetching %s" % self._config["packages-url"])
         packages_file = piupartslib.open_packages_url(self._config["packages-url"])
         self._binary_db = piupartslib.packagesdb.PackagesDB()
         self._binary_db.read_packages_file(packages_file)
+        self._binary_db.set_known_circular_depends(known_circular_depends)
         packages_file.close()
 
         if self._config["sources-url"]:
@@ -549,6 +585,7 @@ class Section:
         htmlpage = string.Template(HTML_HEADER + LOG_LIST_BODY_TEMPLATE + HTML_FOOTER)
         f = file(filename, "w")
         f.write(htmlpage.safe_substitute( {
+                    "page_title": html_protect(title+" in "+self._config.section),
                     "section_navigation": create_section_navigation(self._section_names,self._config.section),
                     "time": time.strftime("%Y-%m-%d %H:%M %Z"),
                     "title": html_protect(title),
@@ -563,24 +600,28 @@ class Section:
 
 
     def print_by_dir(self, output_directory, logs_by_dir):
-        for dir in logs_by_dir:
-            list = []
-            for basename in logs_by_dir[dir]:
+        for vdir in logs_by_dir:
+            vlist = []
+            for basename in logs_by_dir[vdir]:
                 assert basename.endswith(".log")
                 assert "_" in basename
                 package, version = basename[:-len(".log")].split("_")
-                list.append((os.path.join(dir, basename), package, version))
-            self.write_log_list_page(os.path.join(output_directory, dir + ".html"),
-                                title_by_dir[dir], 
-                                desc_by_dir[dir], list)
+                vlist.append((os.path.join(vdir, basename), package, version))
+            self.write_log_list_page(os.path.join(output_directory, vdir + ".html"),
+                                title_by_dir[vdir], 
+                                desc_by_dir[vdir], vlist)
 
     def find_links_to_logs(self, package_name, dirs, logs_by_dir):
         links = []
-        for dir in dirs:
-          for basename in logs_by_dir[dir]:
+        for vdir in dirs:
+          if vdir == "fail":
+            style = " class=\"needs-bugging\""
+          else:
+            style = ""
+          for basename in logs_by_dir[vdir]:
             if basename.startswith(package_name+"_") and basename.endswith(".log"):
               package, version = basename[:-len(".log")].split("_")
-              links.append("<a href=\"/%s\">%s</a>" % (os.path.join(self._config.section, dir, basename),html_protect(version)))
+              links.append("<a href=\"/%s\"%s>%s</a>" % (os.path.join(self._config.section, vdir, basename),style,html_protect(version)))
         return links
 
     def link_to_maintainer_summary(self, maintainer):
@@ -612,7 +653,7 @@ class Section:
           if link_target == package_name:
             link = html_protect(package_name)
           else:
-            link = "unknown-package-or-udeb"
+            link = "unknown-package"
 
         return link
 
@@ -621,7 +662,7 @@ class Section:
         dirs = ""
 
         if state == "successfully-tested":
-          dirs = ["pass", "fixed"]
+          dirs = ["pass"]
         elif state == "failed-testing":
           dirs = ["fail", "bugged"]
         elif state == "cannot-be-tested":
@@ -689,11 +730,22 @@ class Section:
                      rows += file.read(f)
                      f.close()
                      os.unlink(filename)
-     
-            htmlpage = string.Template(HTML_HEADER + SOURCE_PACKAGE_BODY_TEMPLATE + HTML_FOOTER)
+
+            maintainer = os.path.basename(maint_tpl[:-len("_tpl")])
+
+            distrolinks = "<tr class=\"normalrow\"><td class=\"labelcell\">other distributions: </td><td class=\"contentcell2\" colspan=\"5\">"
+            for section in self._section_names:
+              if section != self._config.section:
+                distrolinks += "<a href=\"/"+section+"/maintainer/"+maintainer_subdir(maintainer)+"/"+maintainer+".html\">"+html_protect(section)+"</a> "
+            distrolinks += "</td></tr>"
+
+            htmlpage = string.Template(HTML_HEADER + MAINTAINER_BODY_TEMPLATE + HTML_FOOTER)
             filename = template_path+".html"
             f = file(filename, "w")
             f.write(htmlpage.safe_substitute( {
+               "page_title": html_protect("Status of "+maintainer+" packages in "+self._config.section),
+               "maintainer": html_protect(maintainer+" in "+self._config.section),
+               "distrolinks": distrolinks,
                "section_navigation": create_section_navigation(self._section_names,self._config.section),
                "time": time.strftime("%Y-%m-%d %H:%M %Z"),
                "rows": rows,
@@ -711,6 +763,10 @@ class Section:
         binaryrows = ""
         for binary in binaries.split(", "):
           state = self._binary_db.state_by_name(binary)
+          if state == "unknown":
+            # Don't track udebs and binary packages on other archs. 
+            # The latter is a FIXME which needs parsing the Packages files from other archs too
+            continue
           current_version = self._source_db.get_control_header(source, "Version")
           if state != "circular-dependency" and not "waiting" in state and "dependency" in state:
             state_style="lightalertlabelcell"
@@ -724,32 +780,37 @@ class Section:
           if state in ("failed-testing", "dependency-does-not-exist", "cannot-be-tested"):
             failed = True
 
-        source_state="unknown"
-        if success: source_state="<img src=\"/images/sunny.png\">"
-        if failed:  source_state="<img src=\"/images/weather-severe-alert.png\">"
+        if binaryrows != "":
+          source_state="unknown"
+          if success: source_state="<img src=\"/images/sunny.png\">"
+          if failed:  source_state="<img src=\"/images/weather-severe-alert.png\">"
 
-        sourcerows = "<tr class=\"titlerow\"><td class=\"titlecell\" colspan=\"6\" id=\"%s\">%s in %s</td></tr>" % (source, source, self._config.section)
-        sourcerows += "<tr class=\"normalrow\"><td class=\"labelcell\">Source:</td><td class=\"contentcell2\"><a href=\"http://packages.qa.debian.org/%s\" target=\"_blank\">%s</a></td><td class=\"labelcell\">piuparts summary:</td><td class=\"contentcell2\">%s</td><td class=\"labelcell\">Version:</td><td class=\"contentcell2\">%s</td></tr>" % (source, html_protect(source), source_state, html_protect(source_version))
-        sourcerows += "<tr class=\"normalrow\"><td class=\"labelcell\">Maintainer:</td><td class=\"contentcell2\" colspan=\"5\">%s</td></tr>" % (self.link_to_maintainer_summary(maintainer))
-        if uploaders:
-          sourcerows += "<tr class=\"normalrow\"><td class=\"labelcell\">Uploaders:</td><td class=\"contentcell2\" colspan=\"5\">%s</td></tr>" % (self.link_to_uploaders(uploaders))
+          sourcerows = "<tr class=\"titlerow\"><td class=\"titlecell\" colspan=\"6\" id=\"%s\">%s in %s</td></tr>" % (source, source, self._config.section)
+          sourcerows += "<tr class=\"normalrow\"><td class=\"labelcell\">Source:</td><td class=\"contentcell2\"><a href=\"http://packages.qa.debian.org/%s\" target=\"_blank\">%s</a></td><td class=\"labelcell\">piuparts summary:</td><td class=\"contentcell2\">%s</td><td class=\"labelcell\">Version:</td><td class=\"contentcell2\">%s</td></tr>" % (source, html_protect(source), source_state, html_protect(source_version))
+          sourcerows += "<tr class=\"normalrow\"><td class=\"labelcell\">Maintainer:</td><td class=\"contentcell2\" colspan=\"5\">%s</td></tr>" % (self.link_to_maintainer_summary(maintainer))
+          if uploaders:
+            sourcerows += "<tr class=\"normalrow\"><td class=\"labelcell\">Uploaders:</td><td class=\"contentcell2\" colspan=\"5\">%s</td></tr>" % (self.link_to_uploaders(uploaders))
         
-        source_summary_page_path = os.path.join(self._output_directory, "source", source_subdir(source))
-        if not os.path.exists(source_summary_page_path):
-           os.makedirs(source_summary_page_path)
-        filename = os.path.join(source_summary_page_path, (source + ".html"))
-        htmlpage = string.Template(HTML_HEADER + SOURCE_PACKAGE_BODY_TEMPLATE + HTML_FOOTER)
-        f = file(filename, "w")
-        f.write(htmlpage.safe_substitute( {
-           "section_navigation": create_section_navigation(self._section_names,self._config.section),
-           "time": time.strftime("%Y-%m-%d %H:%M %Z"),
-           "rows": sourcerows+binaryrows,
-        }))
-        f.close()
+          source_summary_page_path = os.path.join(self._output_directory, "source", source_subdir(source))
+          if not os.path.exists(source_summary_page_path):
+             os.makedirs(source_summary_page_path)
+          filename = os.path.join(source_summary_page_path, (source + ".html"))
+          htmlpage = string.Template(HTML_HEADER + SOURCE_PACKAGE_BODY_TEMPLATE + HTML_FOOTER)
+          f = file(filename, "w")
+          f.write(htmlpage.safe_substitute( {
+             "page_title": html_protect("Status of source package "+source+" in "+self._config.section),
+             "section_navigation": create_section_navigation(self._section_names,self._config.section),
+             "time": time.strftime("%Y-%m-%d %H:%M %Z"),
+             "rows": sourcerows+binaryrows,
+          }))
+          f.close()
 
-        # return parsable values
-        if success: source_state = "pass"
-        if failed:  source_state = "fail"
+          # return parsable values
+          if success: source_state = "pass"
+          if failed:  source_state = "fail"
+        else:
+          source_state = "udeb"
+          sourcerows = ""
 
         return sourcerows, binaryrows, source_state, maintainer, uploaders
 
@@ -802,8 +863,9 @@ class Section:
         sources = ""
         for source in self._source_db.get_all_packages():
             (sourcerows, binaryrows, source_state, maintainer, uploaders) = self.create_source_summary(source, logs_by_dir)
-            sources += "%s: %s\n" % (source, source_state)
-            self.create_maintainer_templates_for_source(source, source_state, sourcerows, binaryrows, maintainer, uploaders)
+            if source_state != "udeb":
+              sources += "%s: %s\n" % (source, source_state)
+              self.create_maintainer_templates_for_source(source, source_state, sourcerows, binaryrows, maintainer, uploaders)
  
         write_file(os.path.join(self._output_directory, "sources.txt"), sources)
 
@@ -814,38 +876,68 @@ class Section:
         r('t <- (read.table("'+countsfile+'",sep=",",header=1,row.names=1))')
         r('cname <- c("date",rep(colnames(t)))')
         # here we define how many days we wants stats for (163=half a year)
-        r('v <- t[(nrow(t)-163):nrow(t),0:12]')
+        #r('v <- t[(nrow(t)-163):nrow(t),0:12]')
+        # make graph since day 1  
+        r('v <- t[0:nrow(t),0:12]')
         # thanks to http://tango.freedesktop.org/Generic_Icon_Theme_Guidelines for those nice colors
-        r('palette(c("#4e9a06", "#ef2929", "#73d216", "#d3d7cf", "#5c3566", "#c4a000", "#fce94f", "#a40000", "#888a85", "#2e3436", "#8ae234",  "#729fcf","#204a87"))')
+        r('palette(c("#4e9a06", "#ef2929", "#d3d7cf", "#5c3566", "#c4a000", "#fce94f", "#a40000", "#888a85", "#2e3436", "#729fcf", "#3465a4", "#204a87", "#555753"))')
         r('bitmap(file="'+pngfile+'",type="png16m",width=16,height=9,pointsize=10,res=100)')
-        r('barplot(t(v),col = 1:13, main="Packages per state in '+self._config.section+' (past half year)", xlab="", ylab="Number of packages",space=0.1,border=0)')
+        r('barplot(t(v),col = 1:13, main="Binary packages per state in '+self._config.section+'", xlab="", ylab="Number of binary packages",space=0.1,border=0)')
         r('legend(x="bottom",legend=colnames(t), ncol=2,fill=1:13,xjust=0.5,yjust=0,bty="n")')
-        return "<tr class=\"normalrow\"> <td class=\"contentcell2\" colspan=\"3\"><a href=\"%s\"><img src=\"/%s/%s\" height=\"450\" width=\"800\" alt=\"Package states in the last half year\"></a></td></tr>\n" % ("states.png", self._config.section, "states.png")
+        return "<tr class=\"normalrow\"> <td class=\"contentcell2\" colspan=\"3\"><a href=\"%s\"><img src=\"/%s/%s\" height=\"450\" width=\"800\" alt=\"Binary package states in %s\"></a></td></tr>\n" % ("states.png", self._config.section, "states.png", self._config.section)
 
     def create_and_link_to_analysises(self,state):
         link="<ul>"
-        templates = find_files_with_suffix(self._output_directory,".tpl")
-        for template in templates:
+        for template, linktarget in linktarget_by_template.iteritems():
+          # sucessful logs only have issues and failed logs only have errors
+          # and "(un)owned (files/directories)" and symlink issues should only be reported in sid
           if (state == "failed-testing" and template[-9:] != "issue.tpl") or (state == "successfully-tested" and template[-9:] == "issue.tpl"):
+           if self._config.section == "sid" or ("owned" not in linktarget and "symlink" not in linktarget):
+            substats = ""
 
             tpl = os.path.join(self._output_directory, template)
-            f = file(tpl, "r")
-            analysis = file.read(f)
-            f.close()
-            os.unlink(tpl)
+            try:
+              f = file(tpl, "r")
+              rows = file.read(f)
+              f.close()
+              os.unlink(tpl)
 
-            htmlpage = string.Template(HTML_HEADER + ANALYSIS_BODY_TEMPLATE + HTML_FOOTER)
-            filename = os.path.join(self._output_directory, template[:-len(".tpl")]+".html")
-            f = file(filename, "w")
-            f.write(htmlpage.safe_substitute( {
-               "section_navigation": create_section_navigation(self._section_names,self._config.section),
-               "time": time.strftime("%Y-%m-%d %H:%M %Z"),
-               "rows": analysis,
-             }))
-            f.close()
+              htmlpage = string.Template(HTML_HEADER + ANALYSIS_BODY_TEMPLATE + HTML_FOOTER)
+              filename = os.path.join(self._output_directory, template[:-len(".tpl")]+".html")
+              f = file(filename, "w")
+              f.write(htmlpage.safe_substitute( {
+                 "page_title": html_protect("Packages in state "+state+" "+linktarget),
+                 "section_navigation": create_section_navigation(self._section_names,self._config.section),
+                 "time": time.strftime("%Y-%m-%d %H:%M %Z"),
+                 "rows": rows,
+               }))
+              f.close()
+              if state == "failed-testing":
+                count_bugged = string.count(rows,"/bugged/")
+                count_failed = string.count(rows,"/fail/") 
+                if count_bugged > 0 or count_failed > 0:
+                  substats = ": "
+                if count_bugged > 0:
+                  substats += "%s bugged" % count_bugged
+                if count_bugged > 0 and count_failed > 0:
+                  substats += ", "
+                if count_failed > 0:
+                  substats += "<span id=\"needs-bugging\">%s failed</span>" % count_failed
+              else:
+                  count_passed = string.count(rows,"/pass/")
+                  if count_passed > 0:
+                    substats += ": %s passed" % count_passed
+              link += "<li><a href=%s>%s</a>%s</li>" % (
+                                                                       template[:-len(".tpl")]+".html", 
+                                                                       linktarget,
+                                                                       substats
+                                                       )
+            except:
+              logging.debug("analysis template %s does not exist." % template)
 
-            link += "<li><a href=%s>%s</a></li>" % (template[:-len(".tpl")]+".html", linktarget_by_template[template])
         link += "</ul>"
+        if link == "<ul></ul>": 
+          link = ""
         return link
 
     def write_section_index_page(self,dirs,total_packages):
@@ -853,9 +945,9 @@ class Section:
         for state in self._binary_db.get_states():
             dir_link = ""
             analysis = ""
-            for dir in dirs:
-              if dir in ("pass","fail","bugged") and state_by_dir[dir] == state:
-                dir_link += "<a href='%s.html'>%s</a> logs<br>" % (dir, html_protect(dir))
+            for vdir in dirs:
+              if vdir in ("pass","fail","bugged") and state_by_dir[vdir] == state:
+                dir_link += "<a href='%s.html'>%s</a> logs<br>" % (vdir, html_protect(vdir))
             if state in ("successfully-tested", "failed-testing"):
               analysis = self.create_and_link_to_analysises(state)
             tablerows += ("<tr class=\"normalrow\"><td class=\"contentcell2\"><a href='state-%s.html'>%s</a>%s</td>" +
@@ -865,11 +957,12 @@ class Section:
         try:
           tablerows += self.make_stats_graph();
         except:
-          logging.debug("python-rpy not installed, disabled graphs.")
+          logging.debug("Error generating the graph images, probably python-rpy is not installed, disabling graphs.")
 
         tablerows += "<tr class=\"normalrow\"> <td class=\"labelcell2\">Total</td> <td class=\"labelcell2\" colspan=\"2\">%d</td></tr>\n" % total_packages
         htmlpage = string.Template(HTML_HEADER + SECTION_INDEX_BODY_TEMPLATE + HTML_FOOTER)
         write_file(os.path.join(self._output_directory, "index.html"), htmlpage.safe_substitute( {
+            "page_title": html_protect(self._config.section+" statistics"),
             "section_navigation": create_section_navigation(self._section_names,self._config.section),
             "time": time.strftime("%Y-%m-%d %H:%M %Z"),
             "section": html_protect(self._config.section),
@@ -881,36 +974,36 @@ class Section:
     def write_state_pages(self):
         for state in self._binary_db.get_states():
             logging.debug("Writing page for %s" % state)
-            list = "<ul>\n"
+            vlist = ""
             for package in self._binary_db.get_packages_in_state(state):
-                list += "<li id=\"%s\">%s (%s)" % (
+                vlist += "<li id=\"%s\">%s (%s)" % (
                                          package["Package"],
                                          self.link_to_source_summary(package["Package"]),
                                          html_protect(package["Maintainer"]))
                 if package.dependencies():
-                    list += "\n<ul>\n"
+                    vlist += "\n<ul>\n"
                     for dep in package.dependencies():
-                        list += "<li>dependency %s is %s</li>\n" % \
+                        vlist += "<li>dependency %s is %s</li>\n" % \
                                   (self.link_to_state_page(self._config.section,dep,dep), 
                                   emphasize_reason(html_protect(self._binary_db.state_by_name(dep))))
-                    list += "</ul>\n"
-                list += "</li>\n"
-            list += "</ul>\n"
+                    vlist += "</ul>\n"
+                vlist += "</li>\n"
             htmlpage = string.Template(HTML_HEADER + STATE_BODY_TEMPLATE + HTML_FOOTER)
             write_file(os.path.join(self._output_directory, "state-%s.html" % state), htmlpage.safe_substitute( {
+                                        "page_title": html_protect("Packages in state "+state+" in "+self._config.section),
                                         "section_navigation": create_section_navigation(self._section_names,self._config.section),
                                         "time": time.strftime("%Y-%m-%d %H:%M %Z"),
                                         "state": html_protect(state),
                                         "section": html_protect(self._config.section),
-                                        "list": list
+                                        "list": vlist
                                        }))
 
     def generate_html(self):
         logging.debug("Finding log files")
-        dirs = ["pass", "fail", "bugged", "fixed", "reserved", "untestable"]
+        dirs = ["pass", "fail", "bugged", "reserved", "untestable"]
         logs_by_dir = {}
-        for dir in dirs:
-            logs_by_dir[dir] = find_files_with_suffix(dir, ".log")
+        for vdir in dirs:
+            logs_by_dir[vdir] = find_files_with_suffix(vdir, ".log")
 
         logging.debug("Copying log files")
         copy_logs(logs_by_dir, self._output_directory)
@@ -940,7 +1033,7 @@ class Section:
         self._section_names = section_names
         self._master_directory = os.path.abspath(os.path.join(master_directory, self._config.section))
         if not os.path.exists(self._master_directory):
-            logging.debug("Warning: %s does not exist. Did you ever let the slave work?" % (self._master_directory, self._config.section))
+            logging.debug("Warning: %s did not exist, now created. Did you ever let the slave work?" % (self._master_directory, self._config.section))
             os.mkdir(self._master_directory)
 
         self._output_directory = os.path.abspath(os.path.join(output_directory, self._config.section))
@@ -976,7 +1069,7 @@ def main():
             section.generate_output(master_directory=master_directory,output_directory=output_directory,section_names=section_names)
             sections.append(section)
 
-
+        # main page
         logging.debug("Writing index page")
         # FIXME: I'm sure the next 3 lines can be written more elegant..
         INDEX_BODY = INDEX_BODY_TEMPLATE
@@ -984,9 +1077,18 @@ def main():
           INDEX_BODY = "".join(read_file(os.path.join(output_directory,"index.tpl")))
         htmlpage = string.Template(HTML_HEADER + INDEX_BODY + HTML_FOOTER)
         write_file(os.path.join(output_directory,"index.html"), htmlpage.safe_substitute( {
+                                 "page_title": "About piuparts.d.o and News",
                                  "section_navigation": create_section_navigation(section_names),
                                  "time": time.strftime("%Y-%m-%d %H:%M %Z"),
                               }))
+        # daily report
+        # FIXME:
+        #generate_daily_report();
+
+        # run-parts FIXME
+        # move the scripts to /usr/share/piuparts/sm-tools (or such)
+        # (what is now started via piupartsm/crontab
+
     else:
         logging.debug("Warning: %s does not exist!?! Creating it for you now." % master_directory)
         os.mkdir(master_directory)
