@@ -10,9 +10,9 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 # Public License for more details.
 # 
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 
 
 """Packages database for distributed piuparts processing
@@ -45,7 +45,44 @@ def rfc822_like_header_parse(input):
         else:
             headers.append(line)
     return headers
+ 
+def unique (s):
+    # taken from http://code.activestate.com/recipes/52560/ - thanks to Tim Peters
+    n = len(s)
+    if n == 0:
+      return []  
 
+    u = {}
+    try:
+      for x in s:
+          u[x] = 1
+    except TypeError:
+      del u  # move on to the next method
+    else:
+      return u.keys()
+
+    try:
+      t = list(s)
+      t.sort()
+    except TypeError:
+      del t  # move on to the next method
+    else:
+      assert n > 0
+      last = t[0]
+      lasti = i = 1
+      while i < n:
+          if t[i] != last:
+              t[lasti] = last = t[i]
+              lasti += 1
+          i += 1
+      return t[:lasti]
+
+    # Brute force is all that's left.
+    u = []
+    for x in s:
+      if x not in u:
+          u.append(x)
+    return u
 
 class Package(UserDict.UserDict):
 
@@ -68,22 +105,36 @@ class Package(UserDict.UserDict):
         return depends
 
     def dependencies(self):
-        list = []
+        vlist = []
         for header in ["Depends", "Pre-Depends"]:
             if header in self:
-                list += self._parse_dependencies(header)
-        return list
+                vlist += self._parse_dependencies(header)
+        return vlist
+
+    def depends_with_alts(self, header_name):
+        vlist = []
+        if header_name in self:
+            parser = DependencyParser(self[header_name])
+            vlist += parser.get_dependencies()
+        return vlist
+
+    def prefer_alt_depends(self, header_name,dep_idx,dep):
+        if header_name in self:
+            if header_name not in self._parsed_deps:
+                  self._parse_dependencies(header_name)
+            if self._parsed_deps[header_name][dep_idx]:
+                self._parsed_deps[header_name][dep_idx] = dep.name
 
     def provides(self):
-        list = []
+        vlist = []
         for header in ["Provides"]:
             if header in self:
-                list += self._parse_dependencies(header)
-        return list
+                vlist += self._parse_dependencies(header)
+        return vlist
 
     def is_testable(self):
-        """Are we testable at all? Essential/required/important aren't."""
-        return self.get("Essential", "") != "yes"
+        """Are we testable at all? Required aren't."""
+        return self.get("Priority", "") != "required"
 
     def dump(self, output_file):
         output_file.write("".join(self.headers))
@@ -111,14 +162,14 @@ class LogDB:
         return dircache.listdir(dirname)
         
     def exists(self, pathname):
-	try:
-	    cache = self.exists_cache
-	except AttributeError:
-	    self.exists_cache = {}
-	    cache = self.exists_cache
-	if pathname not in cache:
-             cache[pathname] = os.path.exists(pathname)
-	return cache[pathname]
+        try:
+            cache = self.exists_cache
+        except AttributeError:
+            self.exists_cache = {}
+            cache = self.exists_cache
+        if pathname not in cache:
+            cache[pathname] = os.path.exists(pathname)
+        return cache[pathname]
         
     def open_file(self, pathname, mode):
         return file(pathname, mode)
@@ -137,17 +188,17 @@ class LogDB:
         return False
     
     def any_log_exists(self, package, subdirs):
-	try:
-	    cache = self.basename_cache
-	except AttributeError:
-	    self.basename_cache = {}
-	    cache = self.basename_cache
+        try:
+            cache = self.basename_cache
+        except AttributeError:
+            self.basename_cache = {}
+            cache = self.basename_cache
         package_name = package["Package"]
         for subdir in subdirs:
             for basename in self.listdir(subdir):
-		if basename not in cache:
-		    cache[basename] = basename.split("_", 1)
-		parts = cache[basename]
+                if basename not in cache:
+                    cache[basename] = basename.split("_", 1)
+                parts = cache[basename]
                 if len(parts) == 2 and parts[0] == package_name:
                     return True
         return False
@@ -185,34 +236,34 @@ class PackagesDB:
     _states = [
         "successfully-tested",
         "failed-testing",
-        "fix-not-yet-tested",
         "cannot-be-tested",
-        "essential-required-important",
+        "essential-required",
         "waiting-to-be-tested",
         "waiting-for-dependency-to-be-tested",
         "dependency-failed-testing",
         "dependency-cannot-be-tested",
         "dependency-does-not-exist",
-        "dependency-fix-not-yet-tested",
         "circular-dependency",
         "unknown",
+        "unknown-preferred-alternative",
+        "no-dependency-from-alternatives-exists",
     ]
     
     _dep_state_to_state = {
         "failed-testing": "dependency-failed-testing",
-        "fix-not-yet-tested": "dependency-fix-not-yet-tested",
         "cannot-be-tested": "dependency-cannot-be-tested",
         "waiting-to-be-tested": "waiting-for-dependency-to-be-tested",
-        "waiting-for-dependency-to-be-tested": 
-            "waiting-for-dependency-to-be-tested",
+        "waiting-for-dependency-to-be-tested": "waiting-for-dependency-to-be-tested",
         "dependency-failed-testing": "dependency-failed-testing",
         "dependency-cannot-be-tested": "dependency-cannot-be-tested",
         "dependency-does-not-exist": "dependency-does-not-exist",
         "circular-dependency": "circular-dependency",
-        "dependency-fix-not-yet-tested": "dependency-fix-not-yet-tested",
+        "unknown-preferred-alternative": "unknown-preferred-alternative",
+        "no-dependency-from-alternatives-exists": "dependency-cannot-be-tested",
     }
 
-    def __init__(self, logdb=None):
+    def __init__(self, logdb=None, prefix=None):
+        self.prefix = prefix
         self._packages_files = []
         self._ready_for_testing = None
         self._logdb = logdb or LogDB()
@@ -220,35 +271,39 @@ class PackagesDB:
         self._in_state = None
         self._package_state = {}
         self.set_subdirs(ok="pass", fail="fail", evil="untestable",
-                         reserved="reserved", moreok=["fixed"],
-                         morefail=["bugged"])
+                         reserved="reserved", morefail=["bugged"])
         
-    def set_subdirs(self, ok=None, fail=None, evil=None, reserved=None,
-                    moreok=None, morefail=None):
+    def set_subdirs(self, ok=None, fail=None, evil=None, reserved=None, morefail=None):
+        # Prefix all the subdirs with the prefix
+        if self.prefix:
+            pformat = self.prefix + "/%s"
+        else:
+            pformat = "%s"
         if ok:
-            self._ok = ok
+            self._ok = pformat % ok
         if fail:
-            self._fail = fail
+            self._fail = pformat % fail
         if evil:
-            self._evil = evil
+            self._evil = pformat % evil
         if reserved:
-            self._reserved = reserved
-        if moreok:
-            self._moreok = moreok
+            self._reserved = pformat % reserved
         if morefail:
-            self._morefail = morefail
-        self._all = [self._ok, self._fail, self._evil, self._reserved] + \
-                    self._moreok + self._morefail
+            self._morefail = [pformat % s for s in morefail]
+        self._all = [self._ok, self._fail, self._evil, self._reserved] + self._morefail
            
     def create_subdirs(self):
-        for dir in self._all:
-            if not os.path.exists(dir):
-                os.makedirs(dir)
+        for sdir in self._all:
+            if not os.path.exists(sdir):
+                os.makedirs(sdir)
         
     def read_packages_file(self, input):
         self._packages_files.append(PackagesFile(input))
         self._packages = None
 
+    def set_known_circular_depends(self, known_circular_depends=[]):
+        self._known_circular_depends = []
+        self._known_circular_depends = list(known_circular_depends)
+
     def _find_all_packages(self):
         if self._packages is None:
             self._packages = {}
@@ -259,7 +314,7 @@ class PackagesDB:
                 for provided in p.provides():
                     if provided not in self._packages:
                         self._packages[provided] = p
-    
+
     def _get_recursive_dependencies(self, package, break_circles=True):
         assert self._packages is not None
         deps = []
@@ -283,12 +338,75 @@ class PackagesDB:
             return "successfully-tested"
         if self._logdb.log_exists(package, [self._fail] + self._morefail):
             return "failed-testing"
-        if self._logdb.log_exists(package, self._moreok):
-            return "fix-not-yet-tested"
         if self._logdb.any_log_exists(package, [self._evil]):
             return "cannot-be-tested"
         if not package.is_testable():
-            return "essential-required-important"
+            return "essential-required"
+
+        # First attempt to resolve (still-unresolved) multiple alternative depends
+        # Definitely sub-optimal, but improvement over blindly selecting first one
+        # 1) Prefer first alternative = "essential-required", prefer it
+        # 2) If no "essential-required", prefer first alternative = "successfully-tested"
+        # 3) Otherwise, prefer first alternative =  "waiting-to-be-tested" IF NO REMAINING
+        #    are "unknown/fail"
+        #
+        # Problems:
+        #   a) We will test and fail when >=1 "successfully-tested" but another
+        #      that failed is selected by apt during test run
+        #   b) False positive "Dependency failed/cannot be tested"; however
+        #       more accurately "waiting-for-dependency-to-be-tested"
+
+        state = None
+        for header in ["Depends", "Pre-Depends"]:
+            alt_deps=package.depends_with_alts(header)
+            for d in range(len(alt_deps)):
+                if len(alt_deps[d]) > 1:
+                    alt_found = 0
+                    alt_fails = 0
+                    alt_unknowns = 0
+                    alt_state = None
+                    prefer_alt_score = 0
+                    prefer_alt_idx = 0
+                    prefer_alt = None
+                    for alternative in alt_deps[d]:
+                        dep = alternative.name
+                        if dep in self._package_state:
+                            alt_found += 1
+                            altdep_state = self._package_state[dep]
+                            if prefer_alt_score < 3 and altdep_state == "essential-required":
+                                prefer_alt = alternative
+                                prefer_alt_idx = d
+                                prefer_alt_score = 3
+                            elif prefer_alt_score < 2 and altdep_state == "successfully-tested":
+                                prefer_alt = alternative
+                                prefer_alt_idx = d
+                                prefer_alt_score = 2
+                            elif prefer_alt_score < 1 and \
+                                 altdep_state in ["waiting-to-be-tested", "waiting-for-dependency-to-be-tested"]:
+                                prefer_alt = alternative
+                                prefer_alt_idx = d
+                                prefer_alt_score = 1
+                            elif altdep_state == "unknown":
+                                alt_unknowns += 1
+                            else:
+                                alt_fails += 1
+                                if alt_state is None:
+                                    alt_state = altdep_state
+
+                    if prefer_alt_score >= 2:
+                        package.prefer_alt_depends(header,prefer_alt_idx,prefer_alt)
+                    elif prefer_alt_score == 1 and ((alt_unknowns + alt_fails) == 0):
+                        package.prefer_alt_depends(header,prefer_alt_idx,prefer_alt)
+                    elif alt_found == 0:
+                        return "no-dependency-from-alternatives-exists"
+                    else:
+                        if alt_state is not None and alt_unknowns == 0:
+                            state = alt_state
+                        elif state is None:
+                            state = "unknown-preferred-alternative"
+
+        if state is not None:
+             return state
 
         for dep in package.dependencies():
             if dep not in self._package_state:
@@ -302,22 +420,41 @@ class PackagesDB:
         state = "waiting-to-be-tested"
         for dep in package.dependencies():
             if self._package_state[dep] not in \
-               ["successfully-tested", "essential-required-important"]:
+               ["successfully-tested", "essential-required"]:
                 state = "unknown"
                 break
         if state == "waiting-to-be-tested":
             return state
 
         deps = self._get_recursive_dependencies(package, break_circles=False)
+        # ignore those packages:
+        for pkg in self._known_circular_depends:
+            if pkg in deps:
+                deps.remove(pkg)
         if package["Package"] in deps:
-            return "circular-dependency"
-            
-        return "unknown"
+            return "circular-dependency" # actually, it's a unknown circular-dependency
+     
+        # treat circular-dependencies as testable (for the part of the circle)
+        state = "unknown" 
+        if package["Package"] in self._known_circular_depends:
+          for dep in package.dependencies():
+            if dep not in self._known_circular_depends and self._package_state[dep] not in \
+               ["successfully-tested", "essential-required"]:
+                state = "unknown"
+                break
+            if dep in self._known_circular_depends and self._package_state[dep] not in \
+               ["failed-testing","dependency-failed-testing"]:
+                state = "waiting-to-be-tested"
+                continue
+        return state
 
     def _compute_package_states(self):
         if self._in_state is not None:
             return
     
+        todo = []
+        unpreferred_alt = []
+
         self._find_all_packages()
         package_names = self._packages.keys()
 
@@ -332,13 +469,17 @@ class PackagesDB:
         while package_names:
             todo = []
             done = []
+            unpreferred_alt = []
             for package_name in package_names:
                 package = self._packages[package_name]
-                if self._package_state[package_name] == "unknown":
+                if self._package_state[package_name] in \
+                   [ "unknown", "unknown-preferred-alternative" ]:
                     state = self._compute_package_state(package)
                     assert state in self._states
                     if state == "unknown":
                         todo.append(package_name)
+                    elif state == "unknown-preferred-alternative":
+                        unpreferred_alt.append(package_name)
                     else:
                         self._in_state[state].append(package_name)
                         self._package_state[package_name] = state
@@ -348,36 +489,71 @@ class PackagesDB:
                 # to do anything the next time either.
                 break
             package_names = todo
+            package_names.extend(unpreferred_alt)
 
-        self._in_state["unknown"] = package_names
+        self._in_state["unknown"] = todo
+        self._in_state["unknown-preferred-alternative"] = unpreferred_alt
         
         for state in self._states:
             self._in_state[state].sort()
 
-    def get_total_packages(self):
-        self._find_all_packages()
-        return len(self._packages)
-
     def get_states(self):
         return self._states
 
-    def get_packages_in_state(self, state):
+    def get_pkg_names_in_state(self, state):
         self._compute_package_states()
-        return [self._packages[name] for name in self._in_state[state]]
+        return self._in_state[state]
+    
+    def get_packages_in_state(self, state):
+      self._compute_package_states()
+      return unique([self._packages[name] for name in self._in_state[state]])
+    
+    def get_all_packages(self):
+        self._find_all_packages()
+        return self._packages
+
+    def get_control_header(self, package_name, header):
+        if header == "Source":
+          # binary packages build from the source package with the same name
+          # don't have a Source header, so let's try:
+          try:
+            _source = self._packages[package_name][header]
+            # for binNMU the Source header in Packages files holds the version 
+            # too, so we need to chop it of:
+            if " " in _source:
+              source, version = _source.split(" ")
+            else:
+              source = _source
+          except:
+            source = self._packages[package_name]["Package"]
+          return source
+        elif header == "Uploaders":
+          # not all (source) packages have an Uploaders header
+          uploaders = ""
+          try:
+            uploaders = self._packages[package_name][header]
+          except:
+            pass
+          return uploaders
+        else:
+          return self._packages[package_name][header]
+
+    def get_package_state(self, package_name):
+        return self._package_state[package_name]
 
     def state_by_name(self, package_name):
         if package_name in self._package_state:
             return self._package_state[package_name]
         else:
-            return "unknown-package"
+            return "unknown"
 
     def _find_packages_ready_for_testing(self):
         return self.get_packages_in_state("waiting-to-be-tested")
 
     def reserve_package(self):
-        list = self._find_packages_ready_for_testing()
-        random.shuffle(list)
-        for p in list:
+        plist = self._find_packages_ready_for_testing()
+        random.shuffle(plist)
+        for p in plist:
             if self._logdb.create(self._reserved, p["Package"],
                                   p["Version"], ""):
                 return p
@@ -413,6 +589,8 @@ class PackagesDB:
     def make_package_untestable(self, package, version, log):
         self._check_for_acceptability_as_filename(package)
         self._check_for_acceptability_as_filename(version)
-        if not self._logdb.create(self._evil, package, version, log):
+        if self._logdb.create(self._evil, package, version, log):
+            self._logdb.remove(self._reserved, package, version)
+        else:
             raise Exception("Log file exists already: %s (%s)" %
                                 (package, version))
